@@ -125,7 +125,9 @@ predict.glmmTMB <- function(object,
                             debug=FALSE,
                             ...) {
   ## FIXME: add re.form
-    
+
+  check_dots(..., .action = "warning")
+               
   if (cov.fit) {
       if (!se.fit) message("se.fit set to TRUE because cov.fit = TRUE")
       se.fit <- TRUE
@@ -203,7 +205,7 @@ predict.glmmTMB <- function(object,
     dd$ziPredictCode <- ziPredCode
     assign("data",dd, ee) ## stick this in the appropriate environment
     newObj <- object$obj
-
+    
     ## restore original values to environment of the object
     ## putting add=TRUE first would be more readable,
     ##  but that tickles a bug in R < 4.0.2
@@ -235,27 +237,39 @@ predict.glmmTMB <- function(object,
   ##  bits we need for any of the model frames ...
   tt <- terms(object$modelInfo$allForm$combForm)
   pv <- attr(terms(model.frame(object)),"predvars")
-  attr(tt,"predvars") <- fix_predvars(pv,tt)
+  ## get rid of response variable     
+  attr(tt,"predvars") <- pv[-2] ## was: fix_predvars(pv,tt)
   mf$formula <- RHSForm(tt, as.form=TRUE)
 
-  ## FIXME:: fix_predvars is ugly, and should be refactored.
-  ## the best solution is probably to attach predvars information
-  ## to formulas/terms for individual components
-  ## {conditional, zi, disp} * {fixed, random}
-  ## and fix things downstream, where the actual model matrices
-  ## are constructed.
-  ##
-  ## There's a fairly high chance of breakage with crazy/unforeseen
+  ## fix_predvars (in utils.R) is NO LONGER USED
+  ## We now rely on the 'variables' and 'predvars' attributes matching
+  ## up correctly, **except for the response variable**, from the
+  ## terms of 'combForm' and the model frame, and working with whatever
+  ## newdata= argument is provided.
+  ## Passes existing tests/known cases.
+  ## We should still be on the lookout for crazy/unforeseen
   ## usage of data-dependent bases (e.g. polynomials or splines with
   ## different arguments in different parts of the model ...)
-  ## Can we detect/warn about these?
-  ##
+  ## This could be further improved by making RHSForm()
+  ##  use delete.response() -- handles dropping response from predvars
+  ## Would still need careful testing etc..
+
+       
   if (is.null(newdata)) {
     mf$data <- mc$data ## restore original data
     newFr <- object$frame
   } else {
-    mf$data <- newdata
     mf$na.action <- na.action
+    if (pop_pred) {
+        ## add missing components in newdata
+        ## (placeholder only to avoid error in model frame construction:
+        ##  value shouldn't matter since all b values will be fixed to NA anyway ...)
+        req_vars <- all.vars(RHSForm(formula(object, reOnly = TRUE)))
+        for (fnew in setdiff(req_vars, names(newdata))) {
+            newdata[[fnew]] <- NA
+        }
+    }
+    mf$data <- newdata
     newFr <- eval.parent(mf)
   }
 
@@ -318,6 +332,9 @@ predict.glmmTMB <- function(object,
   ## 'mkTMBStruc' further down.
   yobs <- augFr[[names(omi$respCol)]]
 
+   ## extract smooth information
+   ## NULL if missing
+   old_smooths <- lapply(omi$reTrms, function(x) x[["smooth_info"]])
 
   ## need eval.parent() because we will do eval(mf) down below ...
   TMBStruc <-
@@ -339,7 +356,23 @@ predict.glmmTMB <- function(object,
                                whichPredict=w,
                                REML=omi$REML,
                                map=omi$map,
-                               sparseX=omi$sparseX))
+                               sparseX=omi$sparseX,
+                               old_smooths = old_smooths,
+                               ## don't need priors when predicting ...
+                               priors = NULL)
+                    )
+
+    ## drop rank-deficient columns if necessary
+    for (nm in c("", "zi", "d")) {
+        xnm <- paste0("X", nm)
+        betanm <- paste0("beta", nm)
+        X <- getME(object, xnm)
+        if (prod(dim(X)) > 0 && !is.null(dd <- attr(X, "col.dropped"))) {
+            if (is(X, "Matrix")) xnm <- paste0(xnm, "S")
+            TMBStruc$data.tmb[[xnm]] <- TMBStruc$data.tmb[[xnm]][,-dd]
+            TMBStruc$parameters[[betanm]] <- TMBStruc$parameters[[betanm]][-dd]
+        }
+  }
 
   ## short-circuit
   if(debug) return(TMBStruc)
@@ -379,6 +412,9 @@ predict.glmmTMB <- function(object,
   }
 
   n_orig <- openmp(n = object$modelInfo$parallel)
+  if (openmp_debug()) {
+      cat("predict: setting OpenMP threads to ", n_orig, " on exit\n")
+  }
   on.exit(openmp(n_orig), add = TRUE)
 
   newObj <- with(TMBStruc,
@@ -397,12 +433,12 @@ predict.glmmTMB <- function(object,
   ## set TMB threads to value from original model fit/reset on exit
   if (!is.null(parallel <- object$modelInfo$parallel)) {
     n_orig <- openmp(NULL)
-    if (debug_openmp) cat("resetting TMB threads to ",  parallel, "\n")
+    if (openmp_debug()) cat("resetting TMB threads to ",  parallel, "\n")
     openmp(parallel)
     on.exit(openmp(n = n_orig), add = TRUE)
   }
 
-  if (debug_openmp) cat("TMB threads currently set to ", openmp(NULL), "\n")
+  if (openmp_debug()) cat("TMB threads currently set to ", openmp(NULL), "\n")
   return_eta <- type %in% c("zlink", "link")
   if (!se.fit) {
     rr <- newObj$report(lp)
