@@ -89,10 +89,10 @@ hasRandom <- function(x) {
 ##' @param parm parameter specifier
 ##' @param object fitted glmmTMB object
 ##' @param full include simple dispersion parameter?
-##' @param include_mapped include mapped parameter indices?
+##' @param include_nonest include mapped parameter indices for non-estimated (mapped or rank-deficient/dropped) parameters?
 ##' @noRd
-getParms <- function(parm=NULL, object, full=FALSE, include_mapped = FALSE) {
-    vv <- vcov(object, full=TRUE, include_mapped = include_mapped)
+getParms <- function(parm=NULL, object, full=FALSE, include_nonest = FALSE) {
+    vv <- vcov(object, full=TRUE, include_nonest = include_nonest)
     sds <- sqrt(diag(vv))
     pnames <- names(sds) <- rownames(vv)       ## parameter names (user-facing)
     ee <- object$obj$env
@@ -100,7 +100,7 @@ getParms <- function(parm=NULL, object, full=FALSE, include_mapped = FALSE) {
     ## don't use object$obj$env$random; we want to keep "beta" vals, which may be
     ## counted as "random" if using REML
     drop_rand <- function(x) x[!x %in% c("b", "bzi")]
-    if (!include_mapped) {
+    if (!include_nonest) {
         intnames <- drop_rand(names(ee$last.par))
     } else {
         pl <- ee$parList()
@@ -301,7 +301,7 @@ NULL
 ##' to \code{\link{glmmTMBControl}}, or set \code{options(glmmTMB.cores=[value])},
 ##' to specify that computations should be done in parallel.)
 ##' @seealso \code{\link[TMB]{benchmark}}, \code{\link{glmmTMBControl}}
-##' @return \code{TRUE} or {FALSE} depending on availability of OpenMP
+##' @return \code{TRUE} or \code{FALSE} depending on availability of OpenMP
 ##' @export
 omp_check <- function() {
     .Call("omp_check", PACKAGE="glmmTMB")
@@ -331,12 +331,18 @@ isNullPointer <- function(x) {
 #'
 #' @rdname gt_load
 #' @param oldfit a fitted glmmTMB object
+#' @param update_gauss_disp update \code{betad} from variance to SD parameterization?
 #' @export
-up2date <- function(oldfit) {
+up2date <- function(oldfit, update_gauss_disp = FALSE) {
   openmp(1)  ## non-parallel/make sure NOT grabbing all the threads!
   if (isNullPointer(oldfit$obj$env$ADFun$ptr)) {
       obj <- oldfit$obj
       ee <- obj$env
+
+      pars <- c(grep("last\\.par", names(ee), value = TRUE), "par",
+                "parfull")
+    
+      ## change name of thetaf to psi
       if ("thetaf" %in% names(ee$parameters)) {
           ee$parameters$psi <- ee$parameters$thetaf
           ee$parameters$thetaf <- NULL
@@ -353,6 +359,29 @@ up2date <- function(oldfit) {
           ee2$parameters$psi <- ee2$parameters$thetaf
           ee2$parameters$thetaf <- NULL
       }
+
+    ## prior_ivars, prior_fvars are defined in priors.R
+      if (!"prior_distrib" %in% names(ee$data)) {
+          ## these are DATA_IVECTOR but apparently after processing
+          ##  TMB turns these into numeric ... ??
+          for (v in prior_ivars) ee$data[[v]] <- numeric(0)
+          for (v in prior_fvars) ee$data[[v]] <- numeric(0)
+      }
+
+      ## switch from variance to SD parameterization
+      if (update_gauss_disp &&
+          family(oldfit)$family == "gaussian") {
+          ee$parameters$betad <- ee$parameters$betad/2
+          for (p in pars) {
+              if (!is.null(nm <- names(ee[[p]]))) {
+                  ee[[p]][nm == "betad"] <- ee[[p]][nm == "betad"]/2
+              }
+              if (!is.null(nm <- names(oldfit$fit[[p]]))) {
+                  oldfit$fit[[p]][nm == "betad"] <- oldfit$fit[[p]][nm == "betad"]/2
+              }
+          }
+      }
+
       oldfit$obj <- with(ee,
                        TMB::MakeADFun(data,
                                       parameters,
@@ -370,6 +399,11 @@ up2date <- function(oldfit) {
       ## don't append() or c(), don't want to lose class info
       oldfit$modelInfo$family$dispersion <- 1
   }
+  if (!"priors" %in% names(oldfit$modelInfo)) {
+      ## https://stackoverflow.com/questions/7944809/assigning-null-to-a-list-element-in-r
+      ## n.b. can't use ...$priors <- NULL
+      oldfit$modelInfo["priors"] <- list(NULL)
+  }
   return(oldfit)
 }
 
@@ -378,8 +412,9 @@ up2date <- function(oldfit) {
 #' @param fn partial path to system file (e.g. test_data/foo.rda)
 #' @param verbose print names of updated objects?
 #' @param mustWork fail if file not found?
+#' @param \dots values passed through to \code{up2date}
 #' @export
-gt_load <- function(fn, verbose=FALSE, mustWork = FALSE) {
+gt_load <- function(fn, verbose=FALSE, mustWork = FALSE, ...) {
     sf <- system.file(fn, package = "glmmTMB")
     found_file <- file.exists(sf)
     if (mustWork && !found_file) {
@@ -390,7 +425,7 @@ gt_load <- function(fn, verbose=FALSE, mustWork = FALSE) {
     for (m in L) {
         if (inherits(get(m), "glmmTMB")) {
             if (verbose) cat(m,"\n")
-            assign(m, up2date(get(m)))
+            assign(m, up2date(get(m), ...))
         }
         assign(m, get(m), parent.env(), envir = parent.frame())
     }
@@ -445,23 +480,32 @@ dtruncated_nbinom1 <- function(x, phi, mu, k=0, log=FALSE) {
 ## for matching map names vs nameList components ...
 par_components <- c("beta","betazi","betad","theta","thetazi","psi")
 
-getAllParnames <- function(object, full) {
+## all parameters, including both mapped and rank-dropped
+getParnames <- function(object, full, include_dropped = TRUE, include_mapped = TRUE) {
                            
-  mkNames <- function(tag) {
+  mkNames <- function(tag="") {
       X <- getME(object,paste0("X",tag))
-      if (trivialFixef(nn <- colnames(X),tag)
+      dropped <- attr(X, "col.dropped") %||% numeric(0)
+      ntot <- ncol(X) + length(dropped)
+      if (ntot == ncol(X) || !include_dropped) {
+          nn <- colnames(X)
+      } else {
+          nn <- character(ntot)
+          nn[-dropped] <- colnames(X)
+          nn[ dropped] <- names(dropped)
+      }
+      if (trivialFixef(nn, tag)
           ## if 'full', keep disp even if trivial, if used by family
           && !(full && tag =="d" &&
                (usesDispersion(family(object)$family) && !zeroDisp(object)))) {
           return(character(0))
       }
+      if (tag == "") return(nn)
       return(paste(tag,nn,sep="~"))
   }
 
-  nameList <- setNames(list(colnames(getME(object,"X")),
-                       mkNames("zi"),
-                       mkNames("d")),
-                names(cNames))
+  nameList <- setNames(Map(mkNames, c("", "zi", "d")),
+                         names(cNames))
 
   if(full) {
       ## FIXME: haven't really decided if we should drop the
@@ -488,29 +532,24 @@ getAllParnames <- function(object, full) {
       
   }
 
-    return(nameList)
-}
-
-
-getEstParnames <- function(object, full) {
-    nameList <- getAllParnames(object, full)
-    map <- object$obj$env$map
-    if (length(map)>0) {
-        ## fullNameList for all variables, including mapped vars
-        ## (nameList will get reduced shortly)
-        for (m in seq_along(map)) {
+  if (!include_mapped) {
+     map <- object$obj$env$map
+     if (length(map)>0) {
+         for (m in seq_along(map)) {
             if (length(NAmap <- which(is.na(map[[m]])))>0) {
                 w <- match(names(map)[m],par_components) ##
                 if (length(nameList)>=w) { ## may not exist if !full
                     nameList[[w]] <- nameList[[w]][-NAmap]
                 }
             }
-        }
-    }
-    return(nameList)
+         } ## for (m in seq_along(map))
+     } ## if (length(map) > 0)
+  }
+
+  return(nameList)
 }
 
-## OBSOLETE: delete eventually
+## OBSOLETE (?)
 
 ## reassign predvars to have term vars in the right order,
 ##  but with 'predvars' values inserted where appropriate
@@ -547,9 +586,18 @@ fix_predvars <- function(pv,tt) {
 }
 
 make_pars <- function(pars, ...) {
-    ## FIXME: check for name matches, length matches etc.
     L <- list(...)
     for (nm in names(L)) {
+        unmatched <- setdiff(names(L), unique(names(pars)))
+        if (length(unmatched) > 0) {
+            warning(sprintf("unmatched parameter names: %s",
+                            paste(unmatched, collapse =", ")))
+            next
+        }
+        if ((len1 <- length(L[[nm]])) != (len2 <- sum(names(pars) == nm))) {
+            stop(sprintf("length mismatch in component %s (%d != %d)",
+                         nm, len1, len2))
+        }
         pars[names(pars) == nm] <- L[[nm]]
     }
     return(pars)
@@ -560,7 +608,8 @@ make_pars <- function(pars, ...) {
 ##' See \code{vignette("sim", package = "glmmTMB")} for more details and examples,
 ##' and \code{vignette("covstruct", package = "glmmTMB")}
 ##' for more information on the parameterization of different covariance structures.
-##' 
+##'
+##' @inheritParams glmmTMB
 ##' @param object a \emph{one-sided} model formula (e.g. \code{~ a + b + c}
 ##' (peculiar naming is for consistency with the generic function, which typically
 ##' takes a fitted model object)
@@ -594,7 +643,10 @@ make_pars <- function(pars, ...) {
 simulate_new <- function(object,
                          nsim = 1,
                          seed = NULL,
+                         family = gaussian,
                          newdata, newparams, ..., show_pars = FALSE) {
+    
+    family <- get_family(family)
     if (!is.null(seed)) set.seed(seed)
     ## truncate
     if (length(object) == 3) stop("simulate_new should take a one-sided formula")
@@ -603,11 +655,12 @@ simulate_new <- function(object,
     form[[3]] <- form[[2]]
     form[[2]] <- quote(..y)
     ## insert a legal value: 1.0 is OK as long as family != "beta_family"
-    newdata[["..y"]] <- if (!identical(list(...)$family, "beta_family")) 1.0 else 0.5
+    newdata[["..y"]] <- if (family$family == "beta_family") 0.5 else 1.0
     r1 <- glmmTMB(form,
-              data = newdata,
-              ...,
-              doFit = FALSE)
+                  data = newdata,
+                  family = family,
+                  ...,
+                  doFit = FALSE)
 ## construct TMB object, but don't fit it
     r2 <- fitTMB(r1, doOptim = FALSE)
     if (show_pars) return(r2$env$last.par)
@@ -615,3 +668,61 @@ simulate_new <- function(object,
                     c(list(r2$env$last.par), newparams))
     replicate(nsim, r2$simulate(par = pars)$yobs, simplify = FALSE)
 }
+
+set_class <- function(x, cls, prepend = TRUE) {
+    if (is.null(x)) return(NULL)
+    if (!prepend) class(x) <- cls
+    else class(x) <- c(cls, class(x))
+    x
+}
+
+## convert from parameter name to component name or vice versa
+## first name shoudl be em
+compsyn <- c(cond = "", zi = "zi", disp = "d")
+match_names <- function(x, to_parvec = FALSE, prefix = "beta") {
+    if (to_parvec) {
+        ## "cond" -> "theta" etc.
+        return(paste0(prefix, compsyn[x]))
+    } else {
+        ## "beta" -> "cond" etc.
+        x <- gsub(prefix, "", x)
+        return(names(compsyn)[match(x, compsyn)])
+    }
+}
+
+get_family <- function(family) {
+    if (is.character(family)) {
+        if (family=="beta") {
+            family <- "beta_family"
+            warning("please use ",sQuote("beta_family()")," rather than ",
+                    sQuote("\"beta\"")," to specify a Beta-distributed response")
+        }
+        family <- get(family, mode = "function", envir = parent.frame(2))
+    }
+
+    if (is.function(family)) {
+        ## call family with no arguments
+        family <- family()
+    }
+
+    ## FIXME: what is this doing? call to a function that's not really
+    ##  a family creation function?
+    if (is.null(family$family)) {
+      print(family)
+      stop("after evaluation, 'family' must have a '$family' element")
+    }
+    return(family)
+}
+
+## add negative-value check to binomial initialization method
+our_binom_initialize <- function(family) {
+    newtest <- substitute(
+        ## added test for glmmTMB
+        if (any(y<0)) {
+            stop(sprintf('negative values not allowed for the %s family', FAMILY))
+        }
+      , list(FAMILY=family))
+    b0 <- binomial()$initialize
+    b0[[length(b0)+1]] <- newtest
+    return(b0)
+}
