@@ -12,9 +12,10 @@ if (getRversion() < "3.3.0") {
 
 test_that("Fitted and residuals", {
     expect_equal(length(fitted(fm2)), nrow(sleepstudy))
-    expect_equal(mean(fitted(fm2)), 298.507891)
-    expect_equal(mean(residuals(fm2)), 0, tol=1e-5)
+    expect_equal(mean(fitted(fm2)), 298.507891, tolerance = 1e-6)
+    expect_equal(mean(residuals(fm2)), 0, tolerance = 1e-4)
     ## Pearson and response are the same for a Gaussian model
+    ## FIXME: still true after var -> sd shift?  why? scale by SD?
     expect_equal(residuals(fm2,type="response"),
                  residuals(fm2,type="pearson"))
     ## ... but not for Poisson or NB ...
@@ -118,7 +119,7 @@ test_that("anova ML/REML checks", {
     ## FIXME: too slow?
     ## speed up/save so we don't need to skip on CRAN
     fmA1 <- glmmTMB(Reaction ~ Days + (Days | Subject), sleepstudy, REML = TRUE)
-    fmA2 <- glmmTMB(Reaction ~ Days + diag(Days | Subject), sleepstudy, REML = TRUE)
+    suppressWarnings(fmA2 <- glmmTMB(Reaction ~ Days + diag(Days | Subject), sleepstudy, REML = TRUE))
     fmA3 <- glmmTMB(Reaction ~ 1 + (1 | Subject), sleepstudy, REML = TRUE)
     fmA4 <- glmmTMB(Reaction ~ Days + (1 | Subject), sleepstudy, REML = FALSE)
     fmA5 <- glmmTMB(Reaction ~ 1 + (1 | Subject), sleepstudy, REML = FALSE)
@@ -195,7 +196,9 @@ test_that("confint", {
                   .Dim = c(2L, 2L),
                   .Dimnames = list(c("(Intercept)", "Days"),
                                    c("2.5 %", "97.5 %"))),
-        tolerance=1e-6)
+        ## answers changed with var -> SD shift, increased tolerance
+        ##  rather than substituting new values
+        tolerance=1e-3)
     ciw <- confint(fm2, 1:2, method="Wald", estimate=FALSE)
     expect_warning(confint(fm2,type="junk"),
                    "extra arguments ignored")
@@ -224,11 +227,11 @@ structure(c(5.48098713179567, 0.0248163864044954, 183.810584890723,
     ## ... no RE
     ci.prof0 <- confint(fm_noRE, full=TRUE, method="profile", npts=3)
     expect_equal(ci.prof0,
-                 structure(c(238.216039176535, 7.99674863649355, 7.51779308310198,
-                             264.368471102549, 12.8955469713508, 7.93347860201449),
+                 structure(c(238.216039176535, 7.99674863649355, 3.758897,
+                             264.368471102549, 12.8955469713508, 3.966739),
                            .Dim = 3:2, .Dimnames = list(c("(Intercept)", "Days", "d~(Intercept)"),
                                                         c("2.5 %", "97.5 %"))),
-                 tolerance=1e-5)
+                 tolerance=1e-4)
 
     ci.prof <- confint(fm2,parm=1,method="profile", npts=3)
     expect_equal(ci.prof,
@@ -241,8 +244,9 @@ structure(c(5.48098713179567, 0.0248163864044954, 183.810584890723,
     expect_equal(ci.uni,
                  structure(c(237.68071,265.12949,251.4050979),
                         .Dim = c(1L, 3L),
-        .Dimnames = list("(Intercept)", c("2.5 %", "97.5 %", "Estimate"))),
-                 tolerance=1e-6)
+                        .Dimnames = list("(Intercept)", c("2.5 %", "97.5 %", "Estimate"))),
+                 ## values changed slightly with var -> SD param shift for Gaussian; loosened tolerance
+                 tolerance=1e-3)
     ## check against 'raw' tmbroot
     tmbr <- TMB::tmbroot(fm2$obj,name=1)
     expect_equal(ci.uni[1:2],unname(c(tmbr)))
@@ -317,10 +321,10 @@ test_that("confint with mapped parameters", {
     ## getParms() not exported ...
     ## expect_equal(getParms("beta_", test2), 1:2)
     ## expect_equal(getParms("beta_", test2, include_mapped = TRUE), 1:3)
-    v1 <- vcov(test2, include_mapped = TRUE)
+    v1 <- vcov(test2, include_nonest = TRUE)
     expect_equal(dim(v1$cond), c(3,3))
     expect_true(all(is.na(v1$cond["x",] )))
-    c1 <- confint(test2, parm = "beta_", include_mapped = TRUE)
+    c1 <- confint(test2, parm = "beta_", include_nonest = TRUE)
     expect_equal(nrow(c1), 3)
     expect_equal(unname(unlist(c1["x",])), c(NA_real_, NA_real_, 0))
 
@@ -332,30 +336,30 @@ test_that("confint with mapped parameters", {
     expect_equal(nrow(c3), 4)
     expect_equal(rownames(c3),
                  c("(Intercept)", "z", "Std.Dev.x|A", "Std.Dev.(Intercept)|B"))
-    c4 <- confint(test2, include_mapped = TRUE)
-    expect_equal(confint(test2, include_mapped = TRUE, parm = "theta_"),
+    c4 <- confint(test2, include_nonest = TRUE)
+    expect_equal(confint(test2, include_nonest = TRUE, parm = "theta_"),
                  confint(test2, parm = "theta_"))
     c5 <- confint(test2, parm = "sigma")
 
     ## expect_equal(getParms("theta_", test1), 5L)
     ## expect_equal(getParms("theta_", test1, include_mapped = TRUE), 5:6)
-    v2 <- vcov(test1, include_mapped = TRUE, full = TRUE)
+    v2 <- vcov(test1, include_nonest = TRUE, full = TRUE)
     expect_equal(dim(v2), c(6,6))
     expect_true(all(is.na(v2["theta_1|B.1",])))
 
-    c6 <- confint(test1, include_mapped = TRUE)
+    c6 <- confint(test1, include_nonest = TRUE)
     expect_equal(rownames(c6),
                  c("(Intercept)", "x", "z", "Std.Dev.x|A", "Std.Dev.(Intercept)|B"))
     c7 <- confint(test1, parm = "theta_")
     expect_equal(rownames(c7), "Std.Dev.x|A")
-    c8 <- confint(test1, parm = "theta_", include_mapped = TRUE)
+    c8 <- confint(test1, parm = "theta_", include_nonest = TRUE)
     expect_equal(rownames(c8), c("Std.Dev.x|A", "Std.Dev.(Intercept)|B"))
     expect_equal(unname(c8["Std.Dev.(Intercept)|B", 1:2]), rep(NA_real_, 2))
 })
 
 
 test_that("profile", {
-    p1_th <- profile(fm1,parm="theta_",npts=4)
+    p1_th <- profile(fm1, parm="theta_", npts=4)
     expect_true(all(p1_th$.par=="theta_1|Subject.1"))
     p1_b <- profile(fm1,parm="beta_",npts=4)
     expect_equal(unique(as.character(p1_b$.par)),
@@ -364,7 +368,7 @@ test_that("profile", {
 
 test_that("profile (no RE)", {
     p0_th <- profile(fm_noRE,npts=4)
-    expect_equal(dim(p0_th),c(43,3))
+    expect_equal(dim(p0_th),c(41,3))
 })
 
 test_that("vcov", {
@@ -517,10 +521,11 @@ test_that("confint works for models with dispformula", {
     d2$sd <- "five"
     dat <- rbind(d1, d2)
     m1 <- glmmTMB(x ~ sd + (1|t), dispformula=~sd, data=dat)
-    ref_val <- structure(c(3.14851028784965, 1.30959944530366, 3.25722952319077,
-                           1.46335165911997, 3.20286990552021, 1.38647555221182), .Dim = 2:3,
-                         .Dimnames = list(c("disp.(Intercept)", "disp.sdten"),
-                                          c("2.5 %", "97.5 %", "Estimate")))
+    ref_val <-
+        structure(c(1.57425515082352, 0.654799732331337, 1.62861475468597, 
+                    0.731675819905297, 1.60143495275475, 0.693237776118317), dim = 2:3,
+                  dimnames = list(c("disp.(Intercept)", "disp.sdten"), c("2.5 %", "97.5 %", 
+                                                                         "Estimate")))
     cc <- confint(m1)
     expect_equal(cc[grep("^disp",rownames(cc)),], ref_val, tolerance = 1e-6)
 })
@@ -568,7 +573,11 @@ test_that("trunc nbinom simulation", {
         ## at their starting values
         m1 <- simfun(y~f, family=f, data=dd, beta=c(-40,39))
         expect_equal(fixef(m1)$cond, c(`(Intercept)` = -40, f2 = 39))
-        expect_equal(fitted(m1),c(4.24835425529159e-18, 0.367879441171442))
+        res <- list("truncated_nbinom1" = c(1.44269504088896, 1.6344435754591),
+                    "truncated_nbinom2" = c(1, 1 + exp(-1)))
+        ## values were previously 0, exp(-1) regardless of nbinom1 vs nbinom2 (dispersion param == 1, start value)
+        ## now that response predicts mean of *truncated* distribution, they differ
+        expect_equal(fitted(m1), res[[f]], tolerance = 1e-5)
         ## should NOT get NaN (or zero) for the first group if hack/fix is working
         expect_equal(unname(unlist(simulate(m1,seed=101))),c(1,1))
     }
@@ -609,3 +618,76 @@ test_that("trunc poisson simulation", {
         par(op)
     }
 })
+
+test_that("de novo simulation", {
+    dd <- data.frame(x = 1:10)
+    expect_error(simulate_new(y ~ x), "should take a one-sided")
+    ss <- simulate_new(~ x,
+                 seed = 101,
+                 family = gaussian,
+                 newdata = dd,
+                 newparams = list(beta = 1:2, betad = 0))
+    expect_equal(head(ss[[1]], 2),
+                      c(2.67396350948461, 5.55246185541914))
+})
+
+test_that("de novo simulation with binomial N>1", {
+    dd <- data.frame(x = 1:10)
+    ss <- simulate_new(~ x,
+                 seed = 101,
+                 family = binomial,
+                 weights = rep(10, 10),
+                 newdata = dd,
+                 newparams = list(beta = c(-0.5, 0.1)))
+    expect_equal(head(ss[[1]], 2),
+                      c(3, 2))
+})
+
+test_that("de novo simulation error checking", {
+    dd <- data.frame(x = 1:10)
+    expect_error(simulate_new(~ x,
+                 seed = 101,
+                 family = gaussian,
+                 newdata = dd,
+                 newparams = list(beta = 0)),
+                 "length mismatch in component beta")
+    expect_warning(simulate_new(~ x,
+                 seed = 101,
+                 family = gaussian,
+                 newdata = dd,
+                 newparams = list(beta = rep(0,2),
+                                  junk = 1:3)),
+                 "unmatched parameter names: junk")
+})
+
+
+test_that("weighted residuals", {
+    set.seed(101)
+    data("cbpp", package = "lme4")
+    wts <- sample(1:2, size = nrow(cbpp), replace = TRUE)
+    ## Pearson tested above ...
+    tmbm4 <- glm(incidence ~ period,
+                 data = cbpp, family = poisson, weights = wts)
+    tmbm5 <- glmmTMB(incidence ~ period,
+                     data = cbpp, family = poisson, weights = wts)
+    for  (type in eval(formals(residuals.glmmTMB)$type)) {
+        expect_equal(residuals(tmbm4, type = type),
+                     residuals(tmbm5, type = type),
+                     tolerance = 1e-6)
+    }
+})
+# This test started also giving a warning on os "mac".
+# test_that("bad inversion in vcov", {
+#     skip_on_os(c("windows", "linux"))
+#     d <- readRDS(system.file("test_data", "strengejacke_nasummary.rds",
+#                              package = "glmmTMB"))
+#     m <- glmmTMB(
+#         QoL ~ time + age + x_tv_dm + x_tv_gm + z1_ti + z2_ti + (1 + time | ID) + (1 + x_tv_dm | ID),
+#         data = d,
+#         REML = TRUE
+#     )
+#     ## only fails on some platforms ... this is sufficient for now ... FIXME
+#     if (getRversion() >= "4.3.0") {
+#         expect_true(all(is.na(vcov(m)$cond)))
+#     }
+# })
